/*
 * The Clear BSD License
 * Copyright 2017 NXP
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted (subject to the limitations in the disclaimer below) provided
 * that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of the copyright holder nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * NO EXPRESS OR IMPLIED LICENSES TO ANY PARTY'S PATENT RIGHTS ARE GRANTED BY THIS LICENSE.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "fsl_flexspi_nor_flash.h"

/*******************************************************************************
 * Code
 ******************************************************************************/
#if defined(__CC_ARM) || defined(__GNUC__)
    __attribute__((section(".boot_hdr.conf")))
#elif defined(__ICCARM__)
#pragma location=".boot_hdr.conf"
#endif

#if 1
const flexspi_nor_config_t flash_config =
{
  .memConfig = {
    .tag                  = FLEXSPI_CFG_BLK_TAG,                       // "FCFB" Tag
    .version              = FLEXSPI_CFG_BLK_VERSION,                   // Version
    .readSampleClkSrc     = 3u,  // Sample clock for flash read accesses are generated by the FlexSPI controller
    .csHoldTime           = 3u,                                        // Default hold time
    .csSetupTime          = 3u,                                        // Default setup time
    .columnAddressWidth   = 3u,                                        // No split column and row addresses
    .controllerMiscOption = 0x59,
    .deviceType           = 0x00,
    .sflashPadType        = kSerialFlash_8Pads,
    .serialClkFreq        = kFlexSpiSerialClk_166MHz,
    .lutCustomSeqEnable   = 1,
    .sflashA1Size         = 0x04000000,                                // 64 MB 
    .dataValidTime        = {0x0F, 0x00},
    .busyOffset           = 0x0F,
    .busyBitPolarity      = 0x01,
    .lookupTable = {
          // Read LUTs        
//          FLEXSPI_LUT_SEQ(CMD_SDR,  FLEXSPI_1PAD, 0x03, RADDR_SDR, FLEXSPI_1PAD, 0x18),
//          FLEXSPI_LUT_SEQ(READ_SDR, FLEXSPI_1PAD, 0x08, JMP_ON_CS, FLEXSPI_1PAD, 0x00),
//          FLEXSPI_LUT_SEQ(STOP,     FLEXSPI_1PAD, 0x00, STOP,      FLEXSPI_1PAD, 0x00),

          0x8B1887A0, 0xA7048F10, 0x00000000, 0x00000000,
          0x87008700, 0x87AA8700, 0x87058700, 0x87708700,
          0x8B1887A0, 0xB70B8F10, 0x0000A704, 0x00000000,
          0x87008700, 0x87AA8700, 0x87058700, 0x87AA8700,
          0x87008700, 0x87558700, 0x87028700, 0x87558700,
          0x87008700, 0x87AA8700, 0x87058700, 0x87808700,
          0x87008700, 0x87AA8700, 0x87058700, 0x87AA8700,
          0x87008700, 0x87558700, 0x87028700, 0x87558700,
          0x8B188700, 0x87008F10, 0x00008730, 0x00000000,
          0x87008700, 0x87AA8700, 0x87058700, 0x87A08700,
          0x8B188700, 0xA3808F10, 0x00000000, 0x00000000,
          0x87008700, 0x87AA8700, 0x87058700, 0x87808700,
          0x87008700, 0x87AA8700, 0x87058700, 0x87AA8700,
          0x87008700, 0x87558700, 0x87028700, 0x87558700,
          0x87008700, 0x87AA8700, 0x87058700, 0x87108700,
          0x00000000, 0x00000000, 0x00000000, 0x00000000
          },
     .lutCustomSeq = { 0x00, 0x00, 0x0000,
                       0x02, 0x01, 0x0000,
                       0x02, 0x03, 0x0000,
                       0x04, 0x05, 0x0000,
                       0x02, 0x09, 0x0000,
                       0x04, 0x0B, 0x0000,
                       0x00, 0x00, 0x0000,
                       0x00, 0x00, 0x0000,
                       0x00, 0x00, 0x0000,
                       0x00, 0x00, 0x0000,
                       0x00, 0x00, 0x0000,
                       0x00, 0x00, 0x0000
     }
#if 0
     .lutCustomSeq = {
          0x00000200, 0x00040000, 0x00000001, 0x00000001,
          0x00000000, 0x00000000, 0x00000000, 0x00000000,
          0x00000000, 0x00000000, 0x00000000, 0x00000000,
          0x00000000, 0x00000000, 0x00000000, 0x00000000
          },
#endif
/*
*/


  },
  .pageSize = 512u,
  .sectorSize = 256u * 1024u,
  .ipcmdSerialClkFreq = 1,
  .serialNorType = 1,
//  .blockSize = 256u * 1024u,
};
#else

const unsigned int data[] = {
  0x42464346, 0x56010400, 0x00000000, 0x00030301,
  0x00000100, 0x00000401, 0x00000040, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00070401, 0x00000000, 0x00000000,
  0x00800000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x0A1804EB, 0x26043206, 0x00000000, 0x00000000,
  0x24040405, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000406, 0x00000000, 0x00000000, 0x00000000,
  0x20010401, 0x00000000, 0x00000000, 0x00000000,
  0x08180420, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x081804D8, 0x00000000, 0x00000000, 0x00000000,
  0x08180402, 0x00002004, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000460, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000100, 0x00001000, 0x00000001, 0x00000000,
  0x00010000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000,
};
#endif